var parseDependencies = require('./dependency-parser.js').parseDependencies,
    BuildDiffProcess = require("./build-diff-process.js").BuildDiffProcess,
    readJsonSync = require('read-json-sync'),
    loggerFactory = require("./sl-logger.js").logger,
    expandCommaSeparatedValues = require('./utils').expandCommaSeparatedValues,
    SlJsInfra = require('sl-js-infra').SlJsInfra,
    DefaultValuesResolver = require('./default-values-resolver'),
    Technology = SlJsInfra.Entities.Technology,
    AgentType = SlJsInfra.Entities.AgentType,
    sourceControlProviders = require('./source-control-providers.js');



function CIA() {
}

CIA.prototype.run = function run(buildArguments, callback) {
    console.log('SeaLights Build agent ' + require("../package.json").version);
    var context = this;
    this.setDefaultsArguments(buildArguments);
    var logger = loggerFactory(buildArguments);
    printConfiguration(buildArguments, logger);
    try {
        var packageJsonFile = getPackageJsonFile(buildArguments.workspacepath, logger)
        var agentEventsController = createAgentEventsController(buildArguments, logger);
        agentEventsController.submitAgentStartedEvent(packageJsonFile).then(function () {
            var parsedDependencies = context._tryParseDependenciesForIntegrationBuild(buildArguments);
            var buildMappingProxy = new SlJsInfra.BackendProxy(buildArguments, logger);
            context.runBuildDiffProcess(buildMappingProxy, parsedDependencies, buildArguments, logger, agentEventsController, callback);
        }).catch(function (e) {
            callback(e)
        })
    } catch (e) {
        callback(e);
    }
}

CIA.prototype.runBuildDiffProcess = function(buildMappingProxy, parsedDependencies, buildArguments, logger, agentEventsController, callback) {
    var buildDiffProcess = new BuildDiffProcess(buildArguments, buildMappingProxy, sourceControlProviders, parsedDependencies, agentEventsController, logger);
    buildDiffProcess.run(buildArguments).then(function () {
        shutdownAgent(logger, buildArguments, null, agentEventsController, callback);
    }).catch(function (err) {
        logger.error(err);
        shutdownAgent(logger, buildArguments, err, agentEventsController, callback);
    });
}

CIA.prototype.setDefaultsArguments = function(buildArguments) {
    var defaults = this._createDefaults();
    for (var i in defaults) {
        if (!buildArguments[i]) {
            buildArguments[i] = defaults[i];
        }
    }
    buildArguments.includedFiles = expandCommaSeparatedValues(buildArguments.includedFiles);
    buildArguments.excludedFiles = expandCommaSeparatedValues(buildArguments.excludedFiles);
}

CIA.prototype._tryParseDependenciesForIntegrationBuild = function (buildArguments) {
    var parsedDependencies = [];
    if (buildArguments.dependency) {
        parsedDependencies = parseDependencies(buildArguments.dependency);

    } else if (buildArguments.dependenciesFile) {
        var parsedJson = readJsonSync(buildArguments.dependenciesFile);
        if (!parsedJson || !Array.isArray(parsedJson)) {
            throw 'Invalid file structure';
        }

        parsedJson.forEach(function (depItem) {
            if (depItem.appName && depItem.build && depItem.branch) {
                parsedDependencies.push({
                    appName: depItem.appName.toString(),
                    branch: depItem.branch.toString(),
                    build: depItem.build.toString()
                });
            } else {
                throw 'Invalid dependency item: ' + JSON.stringify(depItem);
            }
        });
    }
    return parsedDependencies;
}

CIA.prototype._createDefaults = function(){
    var defaultValuesResolver = new DefaultValuesResolver();
    return {
        "includedFiles": defaultValuesResolver.getIncludedFiles(),
        "excludedFiles": defaultValuesResolver.getExcludedFiles(),
        "prefixesOfExcludedFiles": process.env["SL_prefixesOfExcludedFiles"] ? process.env["SL_prefixesOfExcludedFiles"] : "~, :, webpack, node_modules",
        "delayShutdownInSeconds":30
    };
}

function printConfiguration(cfg, logger) {
    logger.info("***************************************************************");
    logger.info("Current Configuration: ");
    logger.info("***************************************************************");

    for(p in cfg)
    {
        if (typeof cfg[p] != "function"){
            logger.info(p + ":", cfg[p]);
        }
    }

    logger.info("***************************************************************");

}

function createAgentEventsController(buildArguments, logger){
    var loader = new SlJsInfra.ConfigLoader();
    var agentConfig = loader.loadAgentConfiguration(buildArguments);
    return new SlJsInfra.AgentEventsController(agentConfig, AgentType.BUILD_SCANNER, Technology.NODEJS, true, logger);
}

function getPackageJsonFile(path, logger) {
    try {
        var packageFilePath = SlJsInfra.Utils.FileUtils.findFileUp("package.json", path);
        return require(packageFilePath);
    } catch (e) {
        logger.info("Could not find package.json file in path '%s'");
        return null
    }
}

//TODO: [SLDEV-4709] Verify if we can remove this completely. Should check how SlNodeJS invokes it.
function sendLastLogs(cfg, logger, callback) {
    if (cfg.sendLogs) {
        logger.remoteStream.checkBuffer(true, function (err) {
            callback();
        })
    } else {
        callback();
    }
}

function shutdownAgent(logger, cfg, err, agentEventsController, callback) {
    sendLastLogs(cfg, logger, function () {
        agentEventsController.submitAgentShutdownEvent().then(function () {
            callback(err);
        })
    });}

module.exports.CIA = CIA;
